#!/bin/sh
#
# Script to run a new ephemeral container, wait for it to exit, and repeat forever.
# Stops if the container aborted.
# Stopping the new ephemeral container is awkward; there's no one name to pass to lxc-stop,
# so to stop it, send a SIGINT to the process with pid `cat pidfile`.
#
# Example: 
# given an image 'centos6slave' containing some custom command 'do-work.sh',
# set up two persistantly ephemeral virtual machines, one building and one testing,
# let them run for five minutes, then shut them both down.
#
#  lxc-start-ephemeral-forever --daemon --pid builder.pid -- --orig centos6slave -- do-work.sh --role=build
#  lxc-start-ephemeral-forever --daemon --pid tester.pid  -- --orig centos6slave -- do-work.sh --role=test
#  sleep 600
#  kill -INT `cat builder.pid`
#  kill -INT `cat tester.pid`
#
# As with pidfile(3), pidfile is deleted upon exit.

usage() {
    echo "usage: lxc-start-ephemeral-forever [-h] [--daemon] [--log logfile] [--pid pidfile] -- args..."
}

help() {
    usage
    echo
    echo "Starts an ephemeral (one-off) container, and restarts it if it exits, until killed."
    echo
    echo "Options:"
    echo "--log logfile : redirect stdout and stderr to logfile"
    echo "--daemon      : run in the background"
    echo "--pid pidfile : if --daemon, get pid to kill from given file"
    echo "args...       : arguments to pass to lxc-start-ephemeral"
}

shortoptions='h'
longoptions='help,daemon:,log:,pid:'

DAEMON=
DAEMONARG=
PIDFILE=

cleanup() {
    # Assume lxc-start-ephemeral also gets the signal, and let it clean up the ephemeral container
    # and any mounts.  All we have to do is clean up the pidfile.
    if [ "$DAEMON" ] && [ "$PIDFILE" ]
    then
        # FIXME: really?  We don't need to do kill -INT `cat $PIDFILE` here?
        rm -f "$PIDFILE"
    fi
    exit 1
}

trap cleanup 2 3 15 # SIGTERM SIGINT SIGQUIT

getopt=$(getopt -o $shortoptions --longoptions  $longoptions -- "$@")
if [ $? != 0 ]; then
    usage
    exit 1;
fi

eval set -- "$getopt"

while true; do
    echo looking at $1
    case "$1" in
        -h|--help)
            help
            exit 1
            ;;
        --log)
            shift
            exec > "$1" 2>&1
            shift
            ;;
        --daemon)
            DAEMON=1
            shift
            ;;
        --pid)
            shift
            PIDFILE=$1
            shift
            ;;
        --)
            shift
            break;;
        *)
            echo $1
            usage
            exit 1
            ;;
    esac
done

run() {
    set -x
    while lxc-start-ephemeral "$@"
    do
        # avoid tight loop in case of quick failure
        sleep 1
    done
}

if [ "$DAEMON" ]
then
    run "$@" &
    echo $? > "$PIDFILE"
else
    run "$@"
fi
